package com.diagbot.facade;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.DrugBillNeoDTO;
import com.diagbot.dto.LisBillNeoDTO;
import com.diagbot.dto.PacsBillNeoDTO;
import com.diagbot.entity.node.LIS;
import com.diagbot.entity.node.Medicine;
import com.diagbot.entity.node.PACS;
import com.diagbot.entity.relationship.Medicine_AgeMin;
import com.diagbot.repository.*;
import com.diagbot.vo.BillNeoVO;
import com.diagbot.vo.StandConvert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 图谱facade
 * @author: Mark
 * @time: 2018/8/6 9:11
 */
@Component
public class NeoFacade {

    @Autowired
    MedicineRepository medicineRepository;
    @Autowired
    PacsRepository pacsRepository;
    @Autowired
    LisRepository lisRepository;


    /**
     * 返回药品缓存信息
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'drugCache'")
    public Map<String, List<String>> getDrugCache() {
        Map<String, List<String>> res = new HashMap<>();
        res.put("ddd", new ArrayList<>());
        return res;
    }



    /**
     * 处理开单合理性图谱数据
     * @param billNeoVO
     * @return
     */
    public List<BillNeoDTO> getBillNeo(BillNeoVO billNeoVO) {

        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();

        billNeoDTOs.addAll(getDrugBill(billNeoVO.getDrugList()));
        billNeoDTOs.addAll(getPacsBill(billNeoVO.getPacsList()));
        billNeoDTOs.addAll(getLisBill(billNeoVO.getLisList()));

        for (BillNeoDTO billNeoDTO:billNeoDTOs) {
            System.out.println(billNeoDTO);
        }
        return billNeoDTOs;
    }



    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(List<String> meds) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        DrugBillNeoDTO drugBillNeoDTO;

        MedicineNode medicineNode = new MedicineNode();
        Medicine medicine;

        for (String medname : meds) {
            billNeoDTO = new BillNeoDTO();
            billNeoDTO.setName(medname);

            drugBillNeoDTO = new DrugBillNeoDTO();

            medicine = medicineRepository.findByNameIs(medname);

            if (medicine!=null) {
                drugBillNeoDTO = medicineNode.MedicinetoDrugDTO(medicine);
            }

            billNeoDTO.setDrugBillNeoDTO(drugBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(List<String> pacslist) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        PacsBillNeoDTO pacsBillNeoDTO;

        PACSNode pacsNode = new PACSNode();
        PACS pacs;

        for (String pacsname : pacslist) {
            billNeoDTO = new BillNeoDTO();
            billNeoDTO.setName(pacsname);

            pacsBillNeoDTO = new PacsBillNeoDTO();

            pacs = pacsRepository.findByNameIs(pacsname);

            if (pacs!=null) {
                pacsBillNeoDTO = pacsNode.PacstoPACSDTO(pacs);
            }

            billNeoDTO.setPacsBillNeoDTO(pacsBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    /**
     * 处理化验开单合理性
     */
    public List<BillNeoDTO> getLisBill(List<String> lislist) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        LisBillNeoDTO lisBillNeoDTO;

        LISNode lisNode = new LISNode();
        LIS lis;

        for (String lisname : lislist) {
            billNeoDTO = new BillNeoDTO();
            billNeoDTO.setName(lisname);

            lisBillNeoDTO = new LisBillNeoDTO();

            lis = lisRepository.findByNameIs(lisname);

            if (lis!=null) {
                lisBillNeoDTO = lisNode.ListoLISDTO(lis);
            }

            billNeoDTO.setLisBillNeoDTO(lisBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }



    /**
     * 标准词转换
     *
     * @param standConvert
     *
     * @return  Map<String, Map<String, String>> -->Map<类型, Map<原始词, 标准词>>
     */
    public Map<String, Map<String, String>> standConvert(StandConvert standConvert) {
        // TODO 待处理业务
        return null;
    }

}
